import uuid
from typing import Any, Dict, List, Union

try:
    from typing import TypedDict
except ImportError:
    from typing_extensions import TypedDict

JSONPrimitive = Union[str, int, None, float]
JSONValue = Union[JSONPrimitive, "JSONObject", List["JSONValue"]]
JSONObject = Dict[str, JSONValue]


# {"type": "Integer", "value": 42}
class ProcessVariable(TypedDict):
    """
    Camunda-serialized variable.
    """

    type: str
    value: JSONPrimitive


ProcessVariables = Dict[str, ProcessVariable]
"""
Camunda-serialized variables mapping.
"""

VariablesMapping = Dict[str, Any]
"""
Camunda-deserialized variables mapping.
"""

CamundaId = Union[str, uuid.UUID]
"""
Internal Camunda object ID.
"""



import logging
from typing import Optional

from ..client import Camunda, get_client
from ..types import VariablesMapping
from ..utils import deserialize_variables, serialize_variables
from .parser import Parser

logger = logging.getLogger(__name__)


def evaluate_dmn(
    dmn_key: str,
    *,
    dmn_id: str = "",
    input_values: VariablesMapping,
    client: Optional[Camunda] = None,
) -> VariablesMapping:
    """
    Evaluate the specified DMN table with the given input values.

    If an ID is provided, the DMN table with the given ID is used. Otherwise, the most
    recent DMN table matching the given key will be evaluated.

    The return value is the output deserialized to Python types.
    """
    logger.debug(
        "Evaluating DMN table with key %s and dmn_id %s",
        dmn_key,
        dmn_id,
    )
    client = client or get_client()
    serialized = serialize_variables(input_values)

    logger.debug("Input data: %r", serialized)

    with client:
        id_part = dmn_id if dmn_id else f"key/{dmn_key}"
        result: list = client.post(
            f"decision-definition/{id_part}/evaluate",
            json={"variables": serialized},
            underscoreize=False,
        )

    # TODO: check output with multiple values/strategies - Camunda returns a list
    # of outputs
    output_variables = {}
    for output in result:
        output_variables.update(deserialize_variables(output))
    return output_variables


def get_dmn_parser(
    dmn_key: str, *, dmn_id: str = "", client: Optional[Camunda] = None
) -> Parser:
    client = client or get_client()
    id_part = dmn_id if dmn_id else f"key/{dmn_key}"
    with client:
        xml: str = client.get(f"decision-definition/{id_part}/xml")["dmn_xml"]
    # JSON response from Camunda is utf-8 encoded
    return Parser(xml=xml.encode("utf-8"))



import json
from collections import OrderedDict
from datetime import date, datetime
from typing import Any, Dict, List, Optional, Tuple, Union

import inflection
from dateutil import parser

from .constants import LATEST
from .types import ProcessVariable, ProcessVariables, VariablesMapping


def parse_definition(definition_ref: str) -> Tuple[bool, str]:
    """
    Given a definition reference, parse it to determine if it's a specific version.
    """
    bits = definition_ref.split(":")
    if len(bits) == 2 and bits[1] == LATEST:
        return (True, bits[0])
    return (False, definition_ref)


def noop(val):
    return val


def underscoreize(data: Union[List, Dict, str, None]) -> Union[List, Dict, str, None]:
    if isinstance(data, list):
        return [underscoreize(item) for item in data]

    if isinstance(data, dict):
        new_data = {}
        for key, value in data.items():
            new_key = inflection.underscore(key)
            # variables are dynamic names, can't make assumptions!
            if key == "variables":
                new_data[new_key] = value
            else:
                new_data[new_key] = underscoreize(value)
        return new_data

    return data


TYPE_MAP = {
    bool: ("Boolean", noop),
    date: (
        "String",
        lambda d: d.isoformat(),
    ),  # Date object requires time information, which we don't have
    datetime: ("Date", lambda d: d.isoformat()),
    int: ("Integer", noop),
    float: ("Double", noop),
    str: ("String", noop),
    type(None): ("Null", noop),
    dict: ("Json", json.dumps),
    list: ("Json", json.dumps),
    OrderedDict: ("Json", json.dumps),
}


REVERSE_TYPE_MAP = {
    "date": parser.parse,
    "json": json.loads,
    "integer": int,
    "short": int,
    "long": int,
}


def serialize_variable(value: Any) -> ProcessVariable:
    """
    Serialize a python type to the appropriate Camunda type.
    """
    val_type = type(value)
    if val_type not in TYPE_MAP:
        raise NotImplementedError(f"Type {val_type} is not implemented yet")

    type_name, converter = TYPE_MAP[val_type]
    return {"type": type_name, "value": converter(value)}


def deserialize_variable(variable: ProcessVariable) -> Any:
    """
    Deserialize a Camunda variable type into the appropriate Python type.
    """
    var_type = variable.get("type", "String")
    converter = REVERSE_TYPE_MAP.get(var_type.lower())
    if converter:
        value = converter(variable["value"])
    else:
        value = variable["value"]  # a JSON primitive that maps to proper python objects

    return value


def serialize_variables(variables: Optional[VariablesMapping]) -> ProcessVariables:
    """
    Given a mapping of variables, serialize every value in the mapping.
    """
    if variables is None:
        return {}
    return {key: serialize_variable(value) for key, value in variables.items()}


def deserialize_variables(variables: ProcessVariables) -> VariablesMapping:
    """
    Given a mapping of Camunda variables, deserialize every value in the mapping.
    """
    return {key: deserialize_variable(value) for key, value in variables.items()}



"""
Public Python API to interact with Activiti.
"""

from typing import Any, Dict, Iterable, List, Optional

import requests

from .camunda_models import ProcessDefinition, Task, factory
from .client import get_client
from .types import CamundaId, ProcessVariables
from .utils import deserialize_variable, serialize_variable


def get_process_definitions() -> List[ProcessDefinition]:
    client = get_client()
    response = client.get("process-definition", {"sortBy": "name", "sortOrder": "asc"})
    return factory(ProcessDefinition, response)


def get_start_form_variables(
    process_key: Optional[str] = None, process_id: Optional[str] = None
) -> ProcessVariables:
    """
    Get the start form variables from the Camunda process.

    If defaults are configured in the Camunda process, these will be returned as value.

    A process ID is more specific than a process key, so if both are provided, the
    process ID will be used.
    """
    if not (process_key or process_id):
        raise ValueError("Provide a process key or process ID")

    if process_id:
        endpoint = f"process-definition/{process_id}/form-variables"
    else:
        endpoint = f"process-definition/key/{process_key}/form-variables"

    # TODO: do any necessary type casting
    client = get_client()
    variables = client.get(endpoint, underscoreize=False)

    return variables


def _get_variable(kind: str, id_ref: CamundaId, name: str) -> Any:
    client = get_client()
    path = f"{kind}/{id_ref}/variables/{name}"
    response_data = client.get(
        path, params={"deserializeValue": "false"}, underscoreize=False
    )
    return deserialize_variable(response_data)


def _get_variables(kind: str, id_ref: CamundaId) -> Dict[str, Any]:
    client = get_client()
    path = f"{kind}/{id_ref}/variables"
    response_data = client.get(
        path, params={"deserializeValues": "false"}, underscoreize=False
    )
    variables = {
        name: deserialize_variable(variable) for name, variable in response_data.items()
    }
    return variables


def get_process_instance_variable(instance_id: CamundaId, name: str) -> Any:
    return _get_variable("process-instance", instance_id, name)


def get_all_process_instance_variables(instance_id: CamundaId) -> Dict[str, Any]:
    return _get_variables("process-instance", instance_id)


def get_task_variable(task_id: CamundaId, name: str, default=None) -> Any:
    try:
        return _get_variable("task", task_id, name)
    except requests.HTTPError as exc:
        if exc.response.status_code == 404:  # variable not set
            return default
        raise


def get_task_variables(task_id: CamundaId) -> Dict[str, Any]:
    return _get_variables("task", task_id)


def send_message(
    name: str,
    process_instance_ids: Iterable[CamundaId],
    variables: Optional[Dict[str, Any]] = None,
) -> None:
    """
    Send a BPMN message into running process instances, with optional process variables.

    :param name: Name/ID of the message definition, extract this from the process.
    :param process_instance_ids: an iterable of process instance IDs, can be uuid
      instances or strings.
    :param variables: Optional mapping of ``{name: value}`` process variables. Will be
      serialized as part of the message sending.
    """
    client = get_client()
    variables = (
        {name: serialize_variable(value) for name, value in variables.items()}
        if variables
        else None
    )
    for instance_id in process_instance_ids:
        body = {
            "messageName": name,
            "processInstanceId": instance_id,
            "processVariables": variables or {},
        }
        client.post("message", json=body)


def get_task(
    task_id: CamundaId, check_history=False, factory_cls=Task
) -> Optional[Task]:
    client = get_client()
    try:
        data = client.get(f"task/{task_id}")
    except requests.HTTPError as exc:
        if exc.response.status_code == 404:
            if not check_history:
                return None

            # see if we can get it from the history
            historical = client.get("history/task", {"taskId": task_id})
            if not historical:
                return None

            assert (
                len(historical) < 2
            ), f"Found multiple tasks in the history for ID {task_id}"

            data = historical[0]
            # these properties do not exist in the history API:
            # https://docs.camunda.org/manual/7.11/reference/rest/history/task/get-task-query/
            data.update(
                {
                    "created": data["start_time"],
                    "delegation_state": None,
                    "suspended": False,
                    "form_key": None,  # cannot determine this...
                    "historical": True,
                }
            )

        else:
            raise

    return factory(factory_cls, data)


def complete_task(task_id: CamundaId, variables: dict) -> None:
    client = get_client()
    variables = {name: serialize_variable(value) for name, value in variables.items()}
    client.post(f"task/{task_id}/complete", json={"variables": variables})



from collections import OrderedDict
from datetime import date, datetime

from django_camunda.utils import serialize_variables


def test_serialize_variables():
    variables = {
        "bool": True,
        "date": date(1999, 12, 31),
        "datetime": datetime(1999, 12, 31, 23, 59, 59),
        "integer": 1,
        "float": 1.0,
        "str": "some-string",
        "none": None,
        "dict": {1: 1},
        "list": [1],
        "ordered_dict": OrderedDict([(1, 1)]),
    }

    serialize_variables(variables) == {
        "bool": {"type": "Boolean", "value": True},
        "date": {"type": "String", "value": "1999-12-31"},
        "datetime": {"type": "Date", "value": "1999-12-31T23:59:59"},
        "integer": {"type": "Integer", "value": 1},
        "float": {"type": "Double", "value": 1.0},
        "str": {"type": "String", "value": "some-string"},
        "none": {"type": "Null", "value": None},
        "dict": {"type": "Json", "value": '{"1": 1}'},
        "list": {"type": "Json", "value": "[1]"},
        "ordered_dict": {"type": "Json", "value": '{"1": 1}'},
    }



from django.core.exceptions import ValidationError

import pytest
import requests_mock

from django_camunda.models import CamundaConfig


def test_config_validation():
    config = CamundaConfig(
        root_url="https://some.camunda.nl", rest_api_path="engine-rest/"
    )

    with requests_mock.Mocker() as m:
        m.register_uri(
            "GET",
            "https://some.camunda.nl/engine-rest/version",
            json={"version": "7.11.0"},
        )
        config.clean()


def test_config_validation_failure():
    config = CamundaConfig(
        root_url="https://some.camunda.nl", rest_api_path="engine-rest/"
    )

    with requests_mock.Mocker() as m:
        m.register_uri(
            "GET",
            "https://some.camunda.nl/engine-rest/version",
            json={"error": "BOOM"},
            status_code=500,
        )

        with pytest.raises(ValidationError):
            config.clean()



import logging

from lxml.etree import QName, _Element, fromstring

from .datastructures import (
    DRD,
    Decision,
    DecisionTable,
    DMNIntrospectionResult,
    InputClause,
    OutputClause,
    RequiredDecision,
)

NSMAP = {
    "dmn": "https://www.omg.org/spec/DMN/20191111/MODEL/",
    "camunda": "http://camunda.org/schema/1.0/dmn",
}

logger = logging.getLogger(__name__)


def decision_table_from_xml(node: _Element) -> DecisionTable:
    """
    Build a decision table from
    """
    # id is optional, we currently also don't need references to tables
    table_id = node.get("id", "")

    # extract the inputs
    inputs = [
        input_clause
        for input_element in node.iterfind("dmn:input", namespaces=NSMAP)
        if (input_clause := input_clause_from_xml(input_element))
    ]
    # extract the outputs - the standard says that outputs SHALL have a name if multiple
    # outputs are produced, and SHALL NOT have a name if only a single output is
    # defined. However, Camunda does not enforce this and having a name is pretty
    # useful *actually*. So we only consider named outputs.
    outputs = [
        OutputClause(
            name=name,
            id=output_element.get("id", ""),
            label=output_element.get("label", ""),
            type_ref=output_element.get("typeRef", ""),  # type: ignore
        )
        for output_element in node.iterfind("dmn:output", namespaces=NSMAP)
        if (name := output_element.get("name"))
    ]
    return DecisionTable(id=table_id, inputs=inputs, outputs=outputs)


def input_clause_from_xml(node: _Element) -> InputClause | None:
    """
    Process an input clause node and extract the variable/expression used.
    """
    input_id = node.attrib["id"]
    expression_node = node.find("dmn:inputExpression", namespaces=NSMAP)

    # look up the actual expression
    expression = ""
    if (expression_node is not None) and (
        (text_node := expression_node.find("dmn:text", namespaces=NSMAP)) is not None
    ):
        expression = (text_node.text or "").strip()

    # abort if we can't find an expression
    if not expression:
        logger.debug(
            "Skipping input clause %s, could not find an input expression",
            input_id,
        )
        return None

    return InputClause(
        id=input_id,
        label=node.get("label", ""),
        expression=expression,
        type_ref=expression_node.attrib["typeRef"],  # type: ignore
    )


class Parser:
    """Parse a DMN resource.

    Creating a parser will immediately parse the provided resource, so only create
    instances when there's actual processing required.

    The DMN standard: https://www.omg.org/spec/DMN
    Camunda (at the time of writing) supports parts of the DMN 1.3 spec, see
    https://docs.camunda.org/manual/7.20/reference/dmn/.

    PDF with specification: https://www.omg.org/spec/DMN/1.3/PDF

    Drools also has some solid documentation:
    https://docs.drools.org/latest/drools-docs/drools/DMN/index.html#dmn-drd-components-ref_dmn-models

    The parser only supports local references. We currently do not resolve import
    statements as a naive implementation can lead to security issues (SSR forging etc).
    """

    xml: _Element
    drd: DRD

    def __init__(self, xml: bytes):
        self.xml = fromstring(xml)
        self.drd = self.parse()

    def parse(self) -> DRD:
        """Parse the XML and construct a python DRD object.

        Camunda tends to put all the namespace definitions in the root element, so we
        can grab the nsmap from there.
        """
        decisions_node = self.xml
        tag = QName(decisions_node.tag)
        if not tag.localname == "definitions":
            raise ValueError("The XML file does not appear to be a DMN definition.")

        # Create the top-level resource and parse its contents
        attrs = decisions_node.attrib
        drd = DRD(id=attrs["id"], name=attrs["name"])

        # Find the decisions
        for decision_node in decisions_node.iterfind("dmn:decision", namespaces=NSMAP):
            name = decision_node.attrib["name"]
            decision_id = decision_node.attrib.get("id")
            # the 'id' attribute is apparently optional. Camunda uses this as key, so
            # if we don't know the ID, we cannot target this decision anyway.
            if not decision_id:
                logger.info(
                    "Skipping decision in DRD '%s' without ID, it is not addressable.",
                    drd.id,
                )
                continue

            # does it have a decision table?
            if (
                table_node := decision_node.find("dmn:decisionTable", namespaces=NSMAP)
            ) is None:
                logger.info(
                    "Skipping decision '%s' in DRD '%s' without decision table. "
                    "Currently we don't support other variants of decisions.",
                    decision_id,
                    drd.id,
                )
                continue

            decision_table = decision_table_from_xml(table_node)

            local_references = [
                href[1:]
                for dependency in decision_node.iterfind(
                    "dmn:informationRequirement/dmn:requiredDecision", namespaces=NSMAP
                )
                if (href := dependency.attrib.get("href"))
                # we only support local references, no absolute/relative paths
                if href.startswith("#")
            ]
            required_decisions = [
                RequiredDecision(id_ref=id_ref) for id_ref in local_references
            ]

            drd[decision_id] = Decision(
                id=decision_id,
                name=name,
                decision_table=decision_table,
                required_decisions=required_decisions,
            )

        return drd

    def extract_inputs(self, definition_id: str) -> list[InputClause]:
        """Return the inputs of a decision table.

        Inputs for one table may be outputs provided by a dependency. This takes care
        of only filtering out the inputs that must be provided by the caller to evaluate
        the table.
        """
        decision = self.drd[definition_id]

        all_inputs: list[InputClause] = decision.decision_table.inputs

        # check which outputs are provided and inputs required by decision requirements
        # (these are decision tables that must be evaluated *before* the specified one).
        output_names_from_dependencies: list[str] = []
        for required_decision in decision.required_decisions:
            all_inputs += self.extract_inputs(required_decision.id_ref)
            output_names_from_dependencies += [
                output.name for output in self.extract_outputs(required_decision.id_ref)
            ]

        # XXX: this doesn't catch the cases where the input expression is something like
        # `otherOutput + 10`, for that we'd have to do expression parsing and that's
        # overkill (unless a really good library exists somewhere).
        relevant_inputs = [
            input_clause
            for input_clause in all_inputs
            if input_clause.expression not in output_names_from_dependencies
        ]
        return relevant_inputs

    def extract_outputs(self, definition_id: str) -> list[OutputClause]:
        """Return the outputs of a decision table

        If this table is part of a Decision Requirement Diagram (DRD) and the DRD
        contains multiple decision tables, the tables and their relationship are all
        described in the same XML.

        When you evaluate a decision table with Camunda, if the table has another table
        as a dependency it will be evaluated automatically. Only the outputs of the
        table being run (the 'final' table) will be returned. So if the 'dependency'
        tables have intermediate outputs, these are not returned in the final result.
        """
        decision = self.drd[definition_id]
        assert isinstance(decision, Decision)
        return decision.decision_table.outputs

    def extract_parameters(self, definition_id: str) -> DMNIntrospectionResult:
        return DMNIntrospectionResult(
            inputs=self.extract_inputs(definition_id),
            outputs=self.extract_outputs(definition_id),
        )



"""Data structures to represent DMN elements.

These data structures are created/used by the parser when a ``.dmn`` XML is parsed.

The XML file with the decision definitions contains much more information, we do not
extract everything - only what we actively use is extracted and modelled.
"""

from __future__ import annotations

from collections.abc import Collection
from dataclasses import dataclass, field
from typing import Literal, TypeAlias

CamundaDataType = Literal["string", "integer", "long", "boolean", "date", "double"]


@dataclass
class InputClause:
    # Unique identifier of the decision table input
    id: str
    label: str
    # Specifies how the value of the input clause is generated.
    # We expect FEEL expressions, which is part of the DMN standard.
    # Examples are: `foo` but also `a + b`.
    expression: str
    # The type of the input expression after being evaluated.
    type_ref: CamundaDataType


@dataclass
class OutputClause:
    # all fields are technically optional in the XML, we default to empty strings in
    # those cases.
    id: str
    label: str
    # Used to reference the value of the output.
    name: str
    type_ref: CamundaDataType | Literal[""]


@dataclass
class DecisionTable:
    """
    A single decision table with input and output parameters.
    """

    id: str
    # inputs and outputs are ordered in DMN
    inputs: list[InputClause]
    outputs: list[OutputClause]


@dataclass
class RequiredDecision:
    id_ref: str


@dataclass
class Decision:
    """
    A DMN decision element.

    Camunda support decision tables and literal expressions. We currently do not support
    literal expressions.
    """

    id: str
    name: str
    decision_table: DecisionTable
    required_decisions: Collection[RequiredDecision]


# type alias, because *other* types with an ID can also be recorded.
DRDElement: TypeAlias = Decision  # | DecisionTable


@dataclass
class DRD:
    """
    A single definitions element.

    The definitions element (DRD = definitions resource definition) contains all the
    relevant decisions and their relations.
    """

    id: str
    name: str
    _elements: dict[str, DRDElement] = field(default_factory=dict, repr=False)
    """
    Mapping of element IDs to their element.
    """

    def __contains__(self, item: str | DRDElement) -> bool:
        is_key = isinstance(item, str)
        collection_to_check = (
            self._elements.keys() if is_key else self._elements.values()
        )
        return item in collection_to_check

    def __getitem__(self, element_id: str) -> DRDElement:
        return self._elements[element_id]

    def __setitem__(self, element_id: str, element: DRDElement) -> None:
        self._elements[element_id] = element


@dataclass
class DMNVariable:
    label: str
    type: type
    expression_hint: str = ""


@dataclass
class DMNIntrospectionResult:
    inputs: list[InputClause]
    outputs: list[OutputClause]



# Generated by Django 2.2.25 on 2021-12-20 16:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_camunda", "0004_camundaconfig_enabled"),
    ]

    operations = [
        migrations.AlterField(
            model_name="camundaconfig",
            name="enabled",
            field=models.BooleanField(
                default=True,
                help_text="Global flag to enable/disable Camunda integration.",
                verbose_name="enabled",
            ),
        ),
    ]



import pytest

from django_camunda.dmn.parser import Parser


@pytest.mark.assetname("invoiceClassification.dmn")
def test_extract_inputs_decision_table(binary_asset: bytes):
    parser = Parser(xml=binary_asset)

    inputs = parser.extract_inputs("invoiceClassification")

    assert len(inputs) == 2
    amount, category = inputs

    assert amount.label == "Invoice Amount"
    assert amount.expression == "amount"
    assert category.label == "Invoice Category"
    assert category.expression == "invoiceCategory"


@pytest.mark.assetname("invoiceClassification.dmn")
def test_extract_inputs_decision_table_with_dependencies(binary_asset: bytes):
    parser = Parser(xml=binary_asset)

    inputs = parser.extract_inputs("invoice-assign-approver")

    assert len(inputs) == 2
    amount, category = inputs

    assert amount.label == "Invoice Amount"
    assert amount.expression == "amount"
    assert category.label == "Invoice Category"
    assert category.expression == "invoiceCategory"


@pytest.mark.assetname("invoiceClassification.dmn")
def test_extract_outputs_decision_table(binary_asset: bytes):
    parser = Parser(xml=binary_asset)

    outputs = parser.extract_outputs("invoiceClassification")

    assert len(outputs) == 1
    assert outputs[0].label == "Classification"
    assert outputs[0].name == "invoiceClassification"


@pytest.mark.assetname("invoiceClassification.dmn")
def test_extract_outputs_decision_table_with_dependencies(binary_asset: bytes):
    parser = Parser(xml=binary_asset)

    outputs = parser.extract_outputs("invoice-assign-approver")

    assert len(outputs) == 1
    assert outputs[0].label == "Approver Group"
    assert outputs[0].name == "result"


@pytest.mark.assetname("multi-drd.dmn")
def test_introspect_multi_table_drds_with_dependencies(binary_asset: bytes):
    """
    Assert that the parser resolves `requiredDecision` nodes.
    """
    parser = Parser(binary_asset)

    leaf_inputs = parser.extract_inputs("multi-2")

    input_expressions = {x.expression for x in leaf_inputs}
    assert input_expressions == {"a", "a + b", "reference"}


@pytest.mark.assetname("multi-drd.dmn")
def test_get_full_introspection_result(binary_asset: bytes):
    parser = Parser(binary_asset)

    parameters = parser.extract_parameters("multi-2")

    assert len(parameters.inputs) == 3
    assert len(parameters.outputs) == 1



from django.urls import reverse

import pytest

from django_camunda.models import CamundaConfig


@pytest.mark.django_db
def test_enter_basic_auth(requests_mock, django_app, admin_user):
    config = CamundaConfig.get_solo()
    assert config.auth_header == ""
    django_app.set_user(admin_user)
    requests_mock.get(
        "https://camunda.example.com/engine-rest/version",
        status_code=200,
        json={"version": "7.11.0"},
    )
    admin_url = reverse("admin:django_camunda_camundaconfig_changelist")

    change_page = django_app.get(admin_url)

    change_page.forms["camundaconfig_form"]["basic_auth_username"] = "dummy"
    change_page.forms["camundaconfig_form"]["basic_auth_password"] = "secret"
    change_page.forms["camundaconfig_form"].submit().follow()

    config.refresh_from_db()
    assert config.auth_header == "Basic ZHVtbXk6c2VjcmV0"

    auth = requests_mock.last_request.headers["Authorization"]
    assert auth == "Basic ZHVtbXk6c2VjcmV0"


@pytest.mark.django_db
def test_select_process_definition(requests_mock, django_app, admin_user):
    django_app.set_user(admin_user)
    requests_mock.get(
        "https://camunda.example.com/engine-rest/process-definition",
        json=[
            {
                "id": "dummy:1",
                "category": "http://bpmn.io/schema/bpmn",
                "deployment_id": "1f8bae6e-53e7-11ea-b0b2-7ee96954906c",
                "description": None,
                "diagram": None,
                "history_time_to_live": None,
                "key": "dummy",
                "name": "Dummy",
                "resource": "dummy.bpmn",
                "startable_in_tasklist": True,
                "suspended": False,
                "tenant_id": None,
                "version": 1,
                "version_tag": None,
            },
            {
                "id": "dummy:2",
                "category": "http://bpmn.io/schema/bpmn",
                "deployment_id": "f0bc1357-831b-40a5-859b-21306db1cf08",
                "description": None,
                "diagram": None,
                "history_time_to_live": None,
                "key": "dummy",
                "name": "Dummy",
                "resource": "dummy.bpmn",
                "startable_in_tasklist": True,
                "suspended": False,
                "tenant_id": None,
                "version": 2,
                "version_tag": None,
            },
        ],
    )
    admin_url = reverse("admin:testapp_camunda_add")
    add_page = django_app.get(admin_url)
    field = field = add_page.forms["camunda_form"]["process_definition"]

    try:
        field.select("dummy:2")
    except Exception:
        pytest.fail("Missing option dummy:2")

    with pytest.raises(ValueError):
        field.select("foo")



from django_camunda.dmn import evaluate_dmn


def test_evaluate_by_key(camunda_client, deployed_decision_definition):
    input_vars = {
        "direction": "incoming",
        "port": 443,
    }

    result = evaluate_dmn(
        "firewall_rules", input_values=input_vars, client=camunda_client
    )

    assert result == {"policy": "allow", "reason": "HTTPS traffic"}


def test_evaluate_by_id(camunda_client, deployed_decision_definition):
    input_vars = {
        "direction": "incoming",
        "port": 443,
    }

    result = evaluate_dmn(
        "firewall_rules",
        dmn_id=deployed_decision_definition["id"],
        input_values=input_vars,
        client=camunda_client,
    )

    assert result == {"policy": "allow", "reason": "HTTPS traffic"}
